from django.urls import path, re_path

from . import views

app_name = 'bi'
urlpatterns = [
    # example: /
    path('', views.index, name='index'),
    # example: /reports/
    path('reports/', views.report_list, name='report-list'),
    # example: /reports/Dummy/raw/
    re_path(r'reports/(?P<report_path>.+)/raw/$', views.report_detail_raw, name='report-detail-raw'),
    # example: /reports/Dummy/
    re_path(r'reports/(?P<report_path>.+)/$', views.report_detail, name='report-detail'),
    # example: /dashboards/Dummy/
    path('dashboards/<dashboard_id>/', views.dashboard_detail, name='dashboard-detail'),
    # example: /dashboards/Dummy/DummyReport1/
    path('dashboards/<dashboard_parent_id>/<dashboard_id>/', views.dashboard_detail_nested,
         name='dashboard-detail-nested'),
    # example: /api/flush-cache/
    path('api/flush-cache/', views.flush_cache, name='flush-cache'),
]



from abc import ABC, abstractmethod
from typing import Dict, List, Text

import pandas as pd

from bi.models.object import BaseObject


class BaseDataset(BaseObject, ABC):
    """Base abstract class for all datasets.

    Attributes:
        params: Dataset parameters.
    """

    params: Dict

    def __init__(self, params: Dict = {}) -> None:
        """Inits Dataset.

        Args:
            params: Dataset parameters.
        """
        super().__init__()
        self.params = params

    @abstractmethod
    def get_dataframe(self, params: Dict = None) -> pd.DataFrame:
        """Returns x and y axes data (maybe several).

        Returns:
            Pandas dataframe.
        """
        pass

    @classmethod
    def get_cached_dataset_methods(cls) -> List[Text]:
        """Return all methods cached with cache_dataframe decorator.

        Returns:
            List of methods.
        """
        return [name for name, value in vars(cls).items() if 'cache_dataframe' in str(value)]
